TOW.StylesDesigner = {

	storage: {
		namespace: 'tow.styles',

		load: function () {
			if (Modernizr.localstorage) {
				if (localStorage.getItem(this.namespace) === null) {
					return [];
				}
				return JSON.parse(localStorage.getItem(this.namespace));
			}
		},

		store: function (data) {
			if (Modernizr.localstorage) {
				localStorage[this.namespace] = JSON.stringify(data);
			}
		}
	},


	init: function () {
		this.startup();
		this.initElements();
		this.bindEvents();
	},


	startup: function () {
		this.styles = this.storage.load();
		this.templateLoaded = false;
		this.editMode = false;
	},


	initElements: function () {
		this.$main = $('#main');
		this.$stylesDesignerModal = $('#stylesDesignerModal');
		this.$fm = $('#flashMessages');
		this.$el = this.$stylesDesignerModal;

		// povolit presouvani okna
		this.$stylesDesignerModal.draggable({
			handle: '.modal-header',
			cursor: 'move'
		});
	},


	initTemplateElements: function () {
		this.$btnCreateStyle = $('#btn-create-style');
		this.$btnLoadStyle = $('#btn-load-style');
		this.$btnSaveStyle = $('#btn-save-style');
		this.$btnBack = $('#btn-styles-back');
		this.$btnCreateFile = $('#btnCreateStyleFile');
		this.$btnInsertToEditor = $('#btnInsertStyleToEditor');
		this.$btnCloseModal = $('#btnCloseStylesModal');
		this.$stylesOverview = this.$el.find('#styles-overview');
		this.$stylesEditor = this.$el.find('#styles-editor');
		this.$stylesList = this.$el.find('#styles-list');

		// Style elements

		// Obecne
		this.$inputName = this.$stylesEditor.find('#style-name');

		// Pismo
		this.$fontFace = this.$stylesEditor.find('#font-face');
		this.$fontStyle = this.$stylesEditor.find('#font-style');
		this.$fontSize = this.$stylesEditor.find('#font-size');
		this.$chkSuperscript = this.$stylesEditor.find('#font-superscript');
		this.$chkSubscript = this.$stylesEditor.find('#font-subscript');
		this.$chkCaps = this.$stylesEditor.find('#font-caps');
		this.$chkType = this.$stylesEditor.find('#font-type');

		// Odstavec
		this.$parAlign = this.$stylesEditor.find('#par-align');
		this.$parHspaceLeft = this.$stylesEditor.find('#par-hspace-left');
		this.$parHspaceRight = this.$stylesEditor.find('#par-hspace-right');
		this.$parVspaceBefore = this.$stylesEditor.find('#par-vspace-before');
		this.$parVspaceAfter = this.$stylesEditor.find('#par-vspace-after');
	},


	bindEvents: function () {
		var self = this;

		$('#btnStyles').click(function () {
			if (!self.templateLoaded) {
				// nacteni externi sablony
				$.get('js/templates/styles-designer.hbs', function (data) {
					self.$main.append(data);
					var tpl = Handlebars.compile($('#styles-modal-template').html());
					self.$stylesDesignerModal.find('.modal-body').html(tpl({
						userIsLogged: isUserAutheticated()
					}));
				}).done(function () {
					self.initTemplateElements();
					self.templateLoaded = true;
					self.$btnSaveStyle.hide();
					self.$btnBack.hide();
					self.updateStylesList();
					self.$stylesDesignerModal.toggle('slide', {direction: 'up'}, 'fast');
				});
			} else {
				self.$stylesDesignerModal.toggle('slide', {direction: 'up'}, 'fast');
			}
		});

		this.$el.on('click', '#btn-create-style', this.createStyle.bind(this));
		this.$el.on('click', '#btn-load-style', this.loadStyle.bind(this));
		this.$el.on('click', '#btn-load-style-file', this.loadStyleFile.bind(this));
		this.$el.on('click', '#btn-save-style', this.saveStyle.bind(this));
		this.$el.on('click', '#btn-styles-back', this.backToOverview.bind(this));
		this.$el.on('click', '.style-name', this.editStyle.bind(this));
		this.$el.on('click', '.style-delete', this.deleteStyle.bind(this));
		this.$el.on('click', '.nav a', this.changeNav);

		// ulozit styl do souboru
		this.$el.on('click', '#btnCreateStyleFile', this.onCreateStyleFile.bind(this));

		// vlozit styl do editoru
		this.$el.on('click', '#btnInsertStyleToEditor', this.onInsertStyleToEditor.bind(this));

		// zavrit modal
		this.$el.on('click', '#btnCloseStylesModal', this.closeModal.bind(this));

		// validace nazvu
		this.$el.on('input propertychange', '#style-name', this.onStyleNameChange.bind(this));

		// validace rozmeru
		this.$el.on('input propertychange', '#par-hspace-left, #par-hspace-right, #par-vspace-before, #par-vspace-after', this.onDimensionChange.bind(this));

		// zpracovani udalosti nacteni stylu
		$(document).on('stylesLoaded', function(e, data) {
			self.onStylesFileLoaded(data.content);
		});

		// zpracovani udalosti ulozeni stylu
		$(document).on('stylesSaved', function(e, data) {
			self.onStylesFileSaved(data.styleName);
		});
	},


	onCreateStyleFile: function () {
		var self = this;

		if (!isUserAutheticated()) {
			this.$fm.flashMessage({
				text: 'Pro provedení této akce musíte být přihlášen.',
				className: 'error'
			});
			return;
		}

		var content = this.createStyleContent();
		if (content !== '') {
			this.$stylesDesignerModal.toggle();
			
			// Zobrazit spravce souboru a jeho kontextovou nabidku
			TOW.FileManager.saveStyleMode = true;
			readFiles("");
			initFileMenu("save");
			$( "#sideMenuFiles" ).toggle( "slide", "left", "fast");
			$( "#fileManager" ).toggle( "slide", "left", "fast", setWidthSideContent);
			$( "#file_name" ).focus().val('.sty');
		}
	},


	onStylesFileSaved: function (fileName) {
		this.insertPackage(fileName);
		this.$stylesDesignerModal.toggle('slide', {direction: 'up'}, 'fast');
	},


	onInsertStyleToEditor: function () {
		var content = this.createStyleContent();

		if (!content || content === '') {
			return;
		}

		var Search = ace.require('ace/search').Search;
		var Range = ace.require("ace/range").Range;

		var searchBegin = new Search().set({needle: '%begin-styles%'});
		var rangeBegin = searchBegin.find(editor.getSession());
		var searchEnd = new Search().set({needle: '%end-styles%'});
		var rangeEnd = searchEnd.find(editor.getSession());

		if (!rangeBegin || !rangeEnd) {
			editor.insert(content);
			editor.focus();
		} else {
			var rangeStyles = new Range(rangeBegin.start.row, rangeBegin.start.column, rangeEnd.end.row, rangeEnd.end.column);
			editor.session.replace(rangeStyles, content);
		}
	},


	closeModal: function () {
		this.editMode = false;
		this.$stylesDesignerModal.toggle('slide', {direction: 'up'}, 'fast');
	},


	createStyle: function (e) {
		e.preventDefault();
		this.clearFields();
		this.renderStylesEditor();
	},


	loadStyle: function (e) {
		e.preventDefault();
		var self = this;
		var Search = ace.require('ace/search').Search;
		var Range = ace.require("ace/range").Range;

		var searchBegin = new Search().set({needle: '%begin-styles%'});
		var rangeBegin = searchBegin.find(editor.getSession());
		var searchEnd = new Search().set({needle: '%end-styles%'});
		var rangeEnd = searchEnd.find(editor.getSession());

		if (!rangeBegin || !rangeEnd) {
			self.$fm.flashMessage({
				text: 'Definice stylů nebyla v textu nalezena.',
				className: 'error'
			});
			return;
		}

		var rangeStyles = new Range(rangeBegin.start.row, rangeBegin.start.column, rangeEnd.end.row, rangeEnd.end.column);
		var str = editor.session.getTextRange(rangeStyles);
		self.handleLoadStyle(str);
	},


	loadStyleFile: function (e) {
		e.preventDefault();
		this.$stylesDesignerModal.toggle('slide', {direction: 'up'}, 'fast');
		
		// Zobrazit spravce souboru a jeho kontextovou nabidku
		TOW.FileManager.loadStyleMode = true;
		readFiles('');
		initFileMenu('loadStyles');
		$( "#sideMenuFiles" ).toggle( "slide", "left", "fast");
		$( "#fileManager" ).toggle( "slide", "left", "fast", setWidthSideContent);
	},


	onStylesFileLoaded: function (content) {
		this.handleLoadStyle(content);
		this.$stylesDesignerModal.toggle('slide', {direction: 'up'}, 'fast');
	},


	handleLoadStyle: function (str) {
		console.log(str);
		var self = this;
		var loadedCount = 0;

		try {
			var parser = new Parser();
			var styles = parser.parse(str);
			outer_loop:
				for (var i = 0; i < styles.length; i++) {
					// existuje styl se stejnym nazvem?
					for (var j = 0; j < self.styles.length; j++) {
						if (styles[i].name === self.styles[j].name) {
							if (confirm('Styl ' + styles[i].name + ' už existuje. Chcete jej přepsat?')) {
								self.styles[j] = styles[i];
								loadedCount++;
								self.storage.store(self.styles);
							} else {
								if (confirm('Chcete styl ' + styles[i].name + ' přejmenovat?')) {
									var newFilename = prompt('Zadejte prosím nový název stylu:', styles[i].name);
									
									// zruseni
									if (!newFilename) {
										continue outer_loop;
									}
									
									while (!self.validateName(newFilename) || self.styleExists(newFilename)) {
										var promptText = '';
										if (!self.validateName(newFilename)) {
											promptText = 'Název stylu může obsahovat pouze malá a velká písmena:';
										} else {
											promptText = 'Tento název stylu už existuje, zadejte prosím jiný:';
										}
										
										newFilename = prompt(promptText, newFilename);
										if (!newFilename) {
											continue outer_loop;
										}
									}

									styles[i].name = newFilename;
									self.styles.push(styles[i]);
									loadedCount++;
									self.storage.store(self.styles);
								}
							}
							continue outer_loop;
						}
					}
					self.styles.push(styles[i]);
					loadedCount++;
				}

			if (loadedCount > 0) {
				self.storage.store(self.styles);
				self.$fm.flashMessage({
					text: 'Styly úspěšně nahrány. (počet: ' + loadedCount + ')',
					className: 'success'
				});
			}
			self.updateStylesList();
		} catch (exception) {
			self.$fm.flashMessage({
				text: 'Styly se nepodařilo nahrát, byla nalezena syntaktická chyba.',
				className: 'error'
			});
			alert(exception);
		}
	},


	backToOverview: function (e) {
		e.preventDefault();
		this.editMode = false;
		this.renderStylesOverview();
	},


	renderStylesEditor: function () {
		this.$stylesOverview.hide();
		this.$btnCreateFile.hide();
		this.$btnInsertToEditor.hide();
		this.$btnCloseModal.hide();
		this.$btnSaveStyle.show();
		this.$btnBack.show();
		this.$stylesEditor.show('slide', {direction: 'left'}, 'fast');
	},


	renderStylesOverview: function () {
		this.updateStylesList();
		this.$btnSaveStyle.hide();
		this.$btnBack.hide();
		this.$stylesEditor.hide();
		this.$btnCreateFile.show();
		this.$btnInsertToEditor.show();
		this.$btnCloseModal.show();
		this.$stylesOverview.show('slide', {direction: 'left'}, 'fast');
	},


	saveStyle: function (e) {
		e.preventDefault();
		var self = this;
		var name = this.$inputName.val().trim();
		var fontFace = this.$fontFace.val();
		var fontStyle = this.$fontStyle.val();
		var fontSize = this.$fontSize.val();
		var fontCaps = this.$chkCaps.is(':checked');
		var fontType = this.$chkType.is(':checked');
		var parAlign = this.$parAlign.val();
		var parHspaceLeft = this.$parHspaceLeft.val().replace(/\s/g, '');
		var parHspaceRight = this.$parHspaceRight.val().replace(/\s/g, '');
		var parVspaceBefore = this.$parVspaceBefore.val().replace(/\s/g, '');
		var parVspaceAfter = this.$parVspaceAfter.val().replace(/\s/g, '');

		var style = {
			name: name,
			font: {
				face: fontFace,
				style: fontStyle,
				size: fontSize,
				caps: fontCaps,
				type: fontType
			},
			par: {
				align: parAlign,
				hspaceLeft: parHspaceLeft,
				hspaceRight: parHspaceRight,
				vspaceBefore: parVspaceBefore,
				vspaceAfter: parVspaceAfter
			}
		};

		// validace
		if (!this.validateStyle(style)) {
			return;
		}

		if (this.editMode) {
			$.each(this.styles, function (i, obj) {
				if (obj.name === self.editMode) {
					self.styles[i] = style;
				}
			});
			this.editMode = false;
		} else {
			this.styles.push(style);
		}

		// ulozit styly do local storage
		this.storage.store(this.styles);

		this.$fm.flashMessage({
			text: 'Styl uložen.',
			className: 'success'
		});

		this.renderStylesOverview();
	},


	editStyle: function (e) {
		var name = $(e.target).text();
		var style = null;
		$.each(this.styles, function (i, obj) {
			if (obj.name === name) {
				style = obj;
			}
		});

		if (style) {
			this.editMode = style.name;
			this.setFields(style);
			this.renderStylesEditor();
		}
	},


	deleteStyle: function (e) {
		e.preventDefault();
		var result = confirm('Opravdu chcete smazat tento styl?');
		if (result) {
			var name = $(e.target).data('id');
			this.styles = this.styles.filter(function (el) {
				return el.name !== name;
			});
			this.storage.store(this.styles);
			this.updateStylesList();
		}
	},


	validateStyle: function (style) {
		var error = null;
		var self = this;

		// validace rozmeru
		if (!this.validateDimension(this.$parHspaceLeft.val())) {
			error = 'Zadejte prosím rozměr levého okraje ve správném formátu.';
			this.$parHspaceLeft.focus();
		}

		if (!this.validateDimension(this.$parHspaceRight.val())) {
			error = 'Zadejte prosím rozměr pravého okraje ve správném formátu.';
			this.$parHspaceRight.focus();
		}

		if (!this.validateDimension(this.$parVspaceBefore.val())) {
			error = 'Zadejte prosím rozměr vertikálního odsazení (před) ve správném formátu.';
			this.$parVspaceBefore.focus();
		}

		if (!this.validateDimension(this.$parVspaceAfter.val())) {
			error = 'Zadejte prosím rozměr vertikálního odsazení (za) ve správném formátu.';
			this.$parVspaceAfter.focus();
		}

		// existuje styl se stejnym nazvem?
		$.each(this.styles, function (i, obj) {
			if (obj.name === style.name) {
				if (self.editMode && self.editMode === style.name) {
					return;
				} else {
					error = 'Styl se stejným názvem již existuje.';
				}
			}
		});

		// validace nazvu
		if (!this.validateName(style.name)) {
			error = 'Název stylu může obsahovat pouze písmena bez mezer.';
			this.$inputName.focus();
		}

		// neni zadan nazev stylu?
		if (style.name === '') {
			error = 'Vyplňte prosím název stylu.';
			this.$inputName.focus();
		}

		if (error) {
			this.$fm.flashMessage({
				text: error,
				className: 'error'
			});
			return false;
		}

		return true;
	},


	validateName: function(name) {
		return /^[A-Za-z]+$/.test(name);
	},


	validateDimension: function (dimension) {
		dimension = dimension.replace(/\s/g, '');
		if (dimension === '') {
			return true;
		}
		return /^-?\d+(\.\d+)?(pt|pc|in|bp|cm|mm|dd|cc|sp|ex|em)$/.test(dimension);
	},


	onStyleNameChange: function (e) {
		var value = $(e.target).val();
		var parent = $(e.target).parent();
		var valid = /^[A-Za-z]+$/.test(value);
		if (!valid) {
			parent.addClass('has-error');
		} else {
			parent.removeClass('has-error');
		}
	},


	onDimensionChange: function (e) {
		var value = $(e.target).val();
		var parent = $(e.target).parent();
		if (!this.validateDimension(value)) {
			parent.addClass('has-error');
		} else {
			parent.removeClass('has-error');
		}
	},


	updateStylesList: function () {
		var html = '';
		if (this.styles.length) {
			html += '<table>';
			$.each(this.styles, function (i, obj) {
				html += '<tr>';
				html += '<td class="style-name">' + obj.name + '</td>';
				html += '<td class="style-actions"><a href="#" data-id="' + obj.name + '" class="button modal-button-small red-white-button style-delete">smazat</a></td>';
				html += '</tr>';
			});
			html += '</table>';
		} else {
			html += '<p>Zatím nebyly přidány žádné styly.</p>';
		}

		this.$stylesList.html(html);
	},


	setFields: function (obj) {
		this.$inputName.val(obj.name);
		this.$fontFace.val(obj.font.face);
		this.$fontStyle.val(obj.font.style);
		this.$fontSize.val(obj.font.size);
		this.$chkCaps.prop('checked', obj.font.caps);
		this.$chkType.prop('checked', obj.font.type);
		this.$parAlign.val(obj.par.align);
		this.$parHspaceLeft.val(obj.par.hspaceLeft);
		this.$parHspaceRight.val(obj.par.hspaceRight);
		this.$parVspaceBefore.val(obj.par.vspaceBefore);
		this.$parVspaceAfter.val(obj.par.vspaceAfter);
	},


	clearFields: function () {
		this.$inputName.val('');
		this.$fontFace.val('default');
		this.$fontStyle.val('default');
		this.$fontSize.val('normalsize');
		this.$chkSuperscript.prop('checked', false);
		this.$chkSubscript.prop('checked', false);
		this.$chkCaps.prop('checked', false);
		this.$chkType.prop('checked', false);
		this.$parAlign.val('default');
		this.$parHspaceLeft.val('0em');
		this.$parHspaceRight.val('0em');
		this.$parVspaceBefore.val('0em');
		this.$parVspaceAfter.val('0em');
	},


	changeNav: function (e) {
		e.preventDefault();
		var $el = $(this);
		var $nav = $el.closest('.nav');
		var $tabs = $nav.next('.tab-content');
		$tabs.focus();

		if (!$el.parent('li').hasClass('active')) {
			var tab = $el.data('tab');
			$nav.find('.active').removeClass('active');
			$tabs.find('.active').removeClass('active');
			$el.parent('li').addClass('active');
			$('#tab-' + tab).addClass('active');
		}
	},


	insertPackage: function(package) {
		var Search = ace.require('ace/search').Search;

		// zjistit, zda stejny balik jiz neni vlozeny
		var search = new Search().set({
			needle: '\\usepackage{' + package + '}'
		});
		var range = search.find(editor.session);

		// balik neni vlozeny
		if (!range) {
			// vlozit balik za aktualni usepackage
			search = new Search().set({
				needle: '\\usepackage\{[^\}]*',
				regExp: true
			});
			range = search.find(editor.session);

			// pokud byl usepackage nalezen, vlozit balik za nej, jinak vlozit na prvni radek
			var row = (range) ? range.end.row + 1 : 0;

			editor.session.insert({
				row: row,
				column: 0
			}, '\\usepackage{' + package + '}' + '\n');
		}
		editor.focus();
	},


	styleExists: function(name) {
		for (var i=0; i<this.styles.length; i++) {
			if (this.styles[i].name === name) {
				return true;
			}
		}
		return false;
	},


	parseDimension: function (str) {
		if (str === '') {
			return false;
		}

		// dimension = 0
		var res = str.match(/\d*/);
		if (parseInt(res[0], 10) === 0) {
			return false;
		}

		return str;
	},


	/**
	 * Zpracuje styly a vytvori obsah ze zvolenych parametru
	 */
	createStyleContent: function () {
		var self = this;
		if (!this.styles.length) {
			this.$fm.flashMessage({
				text: 'Není vytvořen žádný styl.',
				className: 'error'
			});
			return;
		}

		var str = '%begin-styles%\n\n';

		$.each(this.styles, function (i, obj) {
			var align = (obj.par.align !== 'default') ? obj.par.align : false;
			var vspaceBefore = self.parseDimension(obj.par.vspaceBefore);
			var vspaceAfter = self.parseDimension(obj.par.vspaceAfter);
			var hspaceLeft = self.parseDimension(obj.par.hspaceLeft);
			var hspaceRight = self.parseDimension(obj.par.hspaceRight);
			var fontSize = (obj.font.size !== 'normalsize') ? obj.font.size : false;
			var fontFace = (obj.font.face !== 'default') ? obj.font.face : false;
			var fontStyle = (obj.font.style !== 'default') ? obj.font.style : false;
			var fontCaps = obj.font.caps;
			var fontType = obj.font.type;

			str += '\\def\\' + obj.name + '#1{\n';

			// zarovnani - start
			if (align) {
				str += '\\begin{' + align + '}\n';
			}

			// mezera pred
			if (vspaceBefore) {
				str += '\\vspace{' + vspaceBefore + '}\n';
			}

			// okraje - start
			if (hspaceLeft || hspaceRight) {
				str += '\\begingroup';
				if (hspaceLeft) {
					str += '\\leftskip=' + hspaceLeft;
				}
				if (hspaceRight) {
					str += '\\rightskip=' + hspaceRight;
				}
				str += '\\par\n';
			}


			// velikost pisma - start
			if (fontSize) {
				str += '{\\' + fontSize;
			}

			// kapitalky - start
			if (fontCaps) {
				str += '\\textsc{';
			}

			// strojopis - start
			if (fontType) {
				str += '\\texttt{';
			}

			// rez pisma - start
			if (fontStyle) {
				str += '\\' + fontStyle + '{';
			}


			// nazev pisma - start
			if (fontFace) {
				str += '{\\fontspec{' + fontFace + '}';
			}

			// OBSAH
			str += '#1';

			// nazev pisma - konec
			if (fontFace) {
				str += '}';
			}

			// rez pisma - konec
			if (fontStyle) {
				str += '}';
			}

			// strojopis - konec
			if (fontType) {
				str += '}';
			}

			// kapitalky - konec
			if (fontCaps) {
				str += '}';
			}

			// velikost pisma - konec
			if (fontSize) {
				str += '}';
			}

			// okraje - konec
			if (hspaceLeft || hspaceRight) {
				str += '\n\\par\\endgroup\n';
			}

			// mezera za
			if (vspaceAfter) {
				str += '\\vspace{' + vspaceAfter + '}\n';
			}

			// zarovnani - konec
			if (align) {
				str += '\\end{' + align + '}\n';
			}

			str += '}\n\n';
		});

		str += '%end-styles%';
		return str;
	}

};



$(function() {
	TOW.StylesDesigner.init();
});
